#include <Keypad.h>
#include <Servo.h>
#include <LiquidCrystal_I2C.h>

const byte ROWS = 4;
const byte COLS = 4;
char keys[ROWS][COLS] = {
  {'1', '2', '3', 'A'},
  {'4', '5', '6', 'B'},
  {'7', '8', '9', 'C'},
  {'*', '0', '#', 'D'}
}; 
byte rowPins[ROWS] = {11, 10, 9, 8};
byte colPins[COLS] = {7, 6, 5, 4};
Keypad keypad = Keypad(makeKeymap(keys), rowPins, colPins, ROWS, COLS);

Servo motor; 
int motorPin = 12;
int motorPosition = 0; 
String enteredDigits = ""; // 用于保存已输入的数字
bool validInput = false; 
unsigned long lastKeyTime; 

LiquidCrystal_I2C lcd(0x27, 16, 2); 

void setup() {
  motor.attach(motorPin);
  motor.write(motorPosition);
  Serial.begin(9600);
  lcd.init(); 
  lcd.backlight(); 
}

void loop() {
  char key = keypad.getKey(); 

  if (key != NO_KEY) { 
    Serial.println(key);
    lastKeyTime = millis();

    if (key == '*') {
      motorPosition = 0; // 设置电机位置为0度
      motor.write(motorPosition); // 移动电机到新位置
    }
    else if (enteredDigits.length() < 4) {
      enteredDigits += key; // 将按键添加到输入数字中
      lcd.clear(); // 清除 LCD 显示
      lcd.setCursor(0, 0);
      lcd.print(enteredDigits); // 在 LCD 上显示输入数字
    }
  }
  
  if (enteredDigits.length() == 4 && millis() - lastKeyTime > 2000) {
    if (enteredDigits == "1234") {
      validInput = true; // 当输入 '0823' 时，标记为有效输入
    } else {
      validInput = false; // 重置有效输入状态
    }
    lcd.clear(); // 清除 LCD 显示
    enteredDigits = ""; // 清空输入数字
  }
  
  if (validInput) { 
    motorPosition = 90; // 设置电机位置为90度
    motor.write(motorPosition); // 移动电机到新位置
    validInput = false; // 重置有效输入状态
  }
}
